#include "UrsUsart1Dma.h"

#include <libmaple/dma.h>
#include <libmaple/usart.h>
#include <libmaple/rcc.h>
#include <libmaple/gpio.h>

dma_tube_reg_map* UrsUsart1DmaClass::channel = DMA1CH4_BASE;   // Zeiger auf DMA1, Channel 4 Register Struktur
usart_dev* usart = USART1;                  // Zeiger auf USART1 Register Struktur
volatile bool UrsUsart1DmaClass::busy = false;

void UrsUsart1DmaClass::initDMA() {
   RCC_BASE->AHBENR |= 1 << 0; // Enable DMA1 Clock

   channel->CCR |= 1 << 4;     // Set the Data Direction: Read from memory
   channel->CCR &= ~(1 << 5);  // Circular mode disabled
   channel->CCR |= 1 << 7;     // Enable the Memory Increment (MINC)
   channel->CCR &= ~(3 << 8);  // Set the Peripheral data size(PSIZE)
   channel->CCR &= ~(3 << 10); // Set the Memory data size (MSIZE) 00 : 8 Bit Data
   channel->CCR &= ~(3 << 12); // Set the Priority Level, PL = 0
   channel->CCR |= 1 << 1;     // Transfer complete interrupt enable
   channel->CPAR = (uint32_t)&usart->regs->DR; //  Set the  peripheral address in PAR Register

   dma_attach_interrupt(DMA1, DMA_CH4, DMA1_Channel4_IRQHandler);
}

void UrsUsart1DmaClass::initUSART1(uint32 baud) {
   RCC_BASE->APB2ENR |= 1 << 2;   // Enable PORTA  Clock
   RCC_BASE->APB2ENR |= 1 << 14;  // Enable USART1 Clock

   // PIN9(TX) Output With 10MHZ Speed, Alternative Func PushPull
   gpio_set_mode(PIN_MAP[PA9].gpio_device, PIN_MAP[PA9].gpio_bit, GPIO_AF_OUTPUT_PP);

/*
Die folgenden drei Anweisungen reichen aus, um die USART nach einem Reset zu initialisieren.
Es ist jedoch ungewiss, welche Werte die Arduino-Bibliothek nach der Verwendung von Serial1 einsetzt.

   usart->regs->CR3 |= 1 << 7; // Enable Usart To Work With DMA As Transmitter (DMAT bit)
   usart->regs->CR1 |= 1 << 3; // Enable The Transmitter (TE Bit) and Enable The USART (UE bit)
   usart->regs->CR1 |= 1 << 13;

Smtliche Register-Werte wurden deshalb einmal nach der korrekten Initialisierung ausgelesen und 
werden nun als komplette Registerinhalte bergeben. Lediglich die Baudrate wird flexibel eingestellt.

USART Register nach der Initialisierung:
CR1:  0x00002008
CR2:  0x00000000
CR3:  0x00000080
GTPR: 0x00000000
*/

   usart->regs->CR1 = 0x00002008;
   usart->regs->CR2 = 0x00000000;
   usart->regs->CR3 = 0x00000080;
   usart->regs->GTPR = 0x00000000;
   usart->regs->SR = 0x00000000;

   // Using The Deafult M Size = 8 bit And 1 Stop bit
   usart_set_baud_rate(usart, USART_USE_PCLK, baud); // Baudrate setzen
}

void UrsUsart1DmaClass::DMA1_Channel4_IRQHandler(void) {
   // Aufrumen:
   DMA1_BASE->IFCR |= 1 << 12; // Channel 4 global interrupt clear
   channel->CCR &= ~(1 << 0);  // Disable The DMA To Work

   busy = false; // bertragung beendet
}

UrsUsart1DmaClass& UrsUsart1DmaClass::getInstance() {
   static UrsUsart1DmaClass instance;
   return instance;
}
void UrsUsart1DmaClass::begin(uint32_t baud) {
   initDMA();
   initUSART1(baud);
}

UrsUsart1DmaClass& ursUsart1Dma = UrsUsart1DmaClass::getInstance();